package org.bukkit.craftbukkit.v1_7_R1;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.ImmutableSet.Builder;
import java.util.Date;
import java.util.Map;
import java.util.Set;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;


public class CraftBanList
  implements org.bukkit.BanList
{
  private final net.minecraft.server.v1_7_R1.BanList list;
  
  public CraftBanList(net.minecraft.server.v1_7_R1.BanList list)
  {
    this.list = list;
  }
  
  public org.bukkit.BanEntry getBanEntry(String target)
  {
    Validate.notNull(target, "Target cannot be null");
    
    if (!list.getEntries().containsKey(target)) {
      return null;
    }
    
    return new CraftBanEntry((net.minecraft.server.v1_7_R1.BanEntry)list.getEntries().get(target), list);
  }
  
  public org.bukkit.BanEntry addBan(String target, String reason, Date expires, String source)
  {
    Validate.notNull(target, "Ban target cannot be null");
    
    net.minecraft.server.v1_7_R1.BanEntry entry = new net.minecraft.server.v1_7_R1.BanEntry(target);
    entry.setSource(StringUtils.isBlank(source) ? entry.getSource() : source);
    entry.setExpires(expires);
    entry.setReason(StringUtils.isBlank(reason) ? entry.getReason() : reason);
    
    list.add(entry);
    list.save();
    return new CraftBanEntry(entry, list);
  }
  
  public Set<org.bukkit.BanEntry> getBanEntries()
  {
    ImmutableSet.Builder<org.bukkit.BanEntry> builder = ImmutableSet.builder();
    for (net.minecraft.server.v1_7_R1.BanEntry entry : list.getEntries().values()) {
      builder.add(new CraftBanEntry(entry, list));
    }
    return builder.build();
  }
  
  public boolean isBanned(String target)
  {
    Validate.notNull(target, "Target cannot be null");
    
    return list.isBanned(target);
  }
  
  public void pardon(String target)
  {
    Validate.notNull(target, "Target cannot be null");
    
    list.remove(target);
  }
}
