package net.mc_pandacraft.java.plugin.pandacraftutils.commands;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.util.StringUtil;

import net.mc_pandacraft.java.plugin.pandacraftutils.PandacraftUtils;

/**
 * Classe parente de toutes les classes devant prendre en charge l'exécution d'une commande Minecraft
 */

public abstract class AbstractCommandExecutor implements CommandExecutor, TabCompleter {
	
	protected PandacraftUtils plugin = PandacraftUtils.getInstance();
	
	protected final String commandName;
	
	/**
	 * 
	 * @param cName (ne peut être null) nom de la commande tel que défini dans le fichier plugin.yml
	 */
	public AbstractCommandExecutor(String cName) {
		if (cName == null) throw new IllegalArgumentException("La commande doit avoir un nom");
		commandName = cName;
		
        plugin.getCommand(commandName).setExecutor(this);
        plugin.getCommand(commandName).setTabCompleter(this);
	}
	
	public String getCommandName() { return commandName; }
	
	public abstract boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args);
	
	public List<String> onTabComplete(CommandSender sender, Command cmd, String alias, String[] args) {
		return null;
	}
	
	/**
	 * <i>Prends en charge les tokens avec des espaces, mais retourne les propositions complets</i>
	 * @param token
	 * @param allProposal
	 * @return
	 */
	public static List<String> getTabProposalFromToken(String token, List<String> allProposal) {
		List<String> ret = new ArrayList<String>();
		
		for (String s : allProposal)
			if (StringUtil.startsWithIgnoreCase(s, token))
				ret.add(s);
		
		if (ret.isEmpty())
			ret.addAll(allProposal);
		
		return ret;
	}

	public static final List<String> NO_PROPOSAL = new ArrayList<String>();
	
	
	
	
	public static boolean isValidPlayerName(String name) {
		if (name == null) return false;
		return name.matches("[0-9a-zA-Z_]{2,16}");
	}
	
	
	/**
	 * <p>Concatène les chaines de caractères passés dans <code>args</code> (avec <code>" "</code> comme séparateur), en ommettant
	 * celles qui se trouvent avant <code>index</code>.<br/>
	 * Par exemple :</p>
	 * <code>
	 * getLastParams(new String[] {"test", "bouya", "chaka", "bukkit"}, 1);
	 * </code>
	 * <p>retournera la chaine "bouya chaka bukkit"
	 * @param args liste des arguments d'une commandes.<br/>
	 * 			Le premier élément est l'argument qui suit le nom de la commande.
	 * 			Usuellement, ce paramètre correspond au paramètre <code>args</code> de la méthode onCommand
	 * @param index 
	 * @return
	 */
	public static String getLastParam(String[] args, int index) {
		if (index < 0 || index >= args.length)
			return null;
		
		
		return StringUtils.join(args, " ", index, args.length);
	}
	
	
}
