package net.mc_pandacraft.java.plugin.pandacraftutils.data_model;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * <b>ORM = Object-Relational Mapping</b><br/>
 * Liste des tables avec leur classes :
 * <ul>
 * 	<li><code>"modo_history" : ModoHistoryTable</code></li>
 * 	<li><code>"staff_ticket" : StaffTicketTable</code></li>
 * 	<li><code>"mp_message" : MPMessageTable</code></li>
 * 	<li><code>"mp_group" : MPGroupTable</code></li>
 * 	<li><code>"mp_group_user" : MPGroupUserTable</code></li>
 * 	<li><code>"mp_web_session" : MPWebSessionTable</code></li>
 * </ul>
 * @author Marc Baloup
 *
 */
public final class ORM {
	
	@SuppressWarnings("rawtypes")
	private static List<SQLTable> tables = new ArrayList<SQLTable>();

	
	public synchronized static void init() {
		try {
			
			/*
			 * Les tables SQL sont à instancier ici !
			 */

			tables.add(new ModoHistoryTable());
			
			tables.add(new StaffTicketTable());

			tables.add(new MPMessageTable());
			tables.add(new MPGroupTable());
			tables.add(new MPGroupUserTable());
			tables.add(new MPWebSessionTable());
			
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	
	
	
	
	@SuppressWarnings("rawtypes")
	public synchronized static <T extends SQLTable> T getTable(Class<T> c) {
		if (c == null) return null;
		for (SQLTable table : tables) {
			
			if (c.isAssignableFrom(table.getClass())) {
				return c.cast(table);
			}
		}
		return null;
	}
	
	
	
	
	
	

	
	private ORM() { } // rend la classe non instanciable
	
	
}
